## Reproduce Table 6.3 from Maronna et al. (2006), page 213
##  Bus data: proportion of unexplained variability for k components
##  of classical and different robust PCA estimates

data(bus)
bus <- as.matrix(bus)

## calculate MADN for each variable
xmad <- apply(bus, 2, mad)
cat("\nMin, Max of MADN: ", min(xmad), which.min(xmad), max(xmad), which.max(xmad), "\n")

## MADN vary between 0 (for variable 9) and 34. Therefore exclude
##  variable 9 and store into 'bus1'.
bus1 <- bus[, -9]

##  Divide the remaining variables by their MADNs and store in 'bus2'.
madbus <- apply(bus1, 2, mad)
bus2 <- sweep(bus1, 2, madbus, "/", check.margin = FALSE)

##  Conducting PCA on bus1 (unscaled) with scale=mad and on bus2
##  (already scaled) with scale=FALSE produce identical results.
##      Please, note: scale=TRUE will produce different results,
##      because by default 'sd' is used.
(pc1 <- PcaClassic(bus1, scale=mad))
(pc2 <- PcaClassic(bus2))
all.equal(pc1$eigenvalues, pc2$eigenvalues)
all.equal(pc1$loadings, pc2$loadings)

## Compute classical and robust PCA (Spherical/Locantore, Hubert, MCD, OGK, MM, GRID and PROJ)
ev    <- getEigenvalues(pca <- PcaClassic(bus2))
evsph <- getEigenvalues(rpcsph <- PcaLocantore(bus2))
evhub <- getEigenvalues(rpchub <- PcaHubert(bus2, k=17, kmax=17, mcd=FALSE))
evmcd <- getEigenvalues(rpcmcd <- PcaCov(bus2, cov.control=CovControlMcd()))
evogk <- getEigenvalues(rpcogk <- PcaCov(bus2, cov.control=CovControlOgk()))
evmm <- getEigenvalues(rpcMM <- PcaCov(bus2, cov.control=CovControlMMest()))
evgrid <- getEigenvalues(rpcgrid <- PcaGrid(bus2))              # by default method="mad" is used,
evproj <- getEigenvalues(rpcproj <- PcaProj(bus2))              # method="qn" will be very slow!

uvar <- cbind.data.frame(k=1:17,
    Classical=(sum(ev)-cumsum(ev))/sum(ev),
    Spherical=(sum(evsph)-cumsum(evsph))/sum(evsph),
    Hubert=(sum(evhub)-cumsum(evhub))/sum(evhub),
    MCD=(sum(evmcd)-cumsum(evmcd))/sum(evmcd),
    OGK=(sum(evogk)-cumsum(evogk))/sum(evogk),
    MM=(sum(evmm)-cumsum(evmm))/sum(evmm),
    GRID=(sum(evgrid)-cumsum(evgrid))/sum(evgrid),
    PROJ=(sum(evproj)-cumsum(evproj))/sum(evproj)
    )
(uvar <- round(uvar,3)[1:6,])

## Reproduce Table 6.4 from Maronna et al. (2006), page 214
##
## Compute classical and robust PCA extracting only the first 3 components
## and take the squared orthogonal distances to the 3-dimensional hyperplane.
## Calculate the quantiles of the distances to the 3-dimensional hyperplane

probs <- seq(0, 1, 0.1)[-c(1,11)]
qq <- rbind(Classic=c(quantile(d <- PcaClassic(bus2, k=3)$od^2, probs=probs), Max=max(d)),
            Spherical=c(quantile(rd1 <- PcaLocantore(bus2, k=3)$od^2, probs=probs), Max=max(d1)),
            Hubert=c(quantile(rd2 <- PcaHubert(bus2, k=3)$od^2, probs=probs), Max=max(d2)),
            MCD=c(quantile(rd3 <- PcaCov(bus2, k=3)$od^2, probs=probs), MAX=max(d3)),
            OGK=c(quantile(rd4 <- PcaCov(bus2, k=3, cov.control=CovControlOgk())$od^2, probs=probs), max(d4)),
            MM=c(quantile(rd5 <- PcaCov(bus2, k=3, cov.control=CovControlMMest())$od^2, probs=probs), max(d5)),
            GRID=c(quantile(rd6 <- PcaGrid(bus2, k=3)$od^2, probs=probs), max(d6)),
            PROJ=c(quantile(rd7 <- PcaProj(bus2, k=3)$od^2, probs=probs), max(d7)))
cat("\nBus data: quantiles of distances to hiperplane\n")
(qq <- round(qq, 1))


## Reproduce Fig 6.1 from Maronna et al. (2006), page 214
##
cat("\nBus data: Q-Q plot of logs of distances to hyperplane (k=3)
from classical and robust estimates.
The line is the identity diagonal.\n")
plot(sort(log(d)), sort(log(rd1)), xlab="classical", ylab="robust")
lines(sort(log(d)), sort(log(d)))
