library(rrcov)

## Example 1: Bus data

## Load the data and standardize with MAD. The data frame 'bus2' will be which we will be using
data(bus)
bus <- as.matrix(bus)

## calculate MAD for each variable
xmad <- apply(bus, 2, mad)
cat("\nMin, Max of MAD: ", min(xmad), max(xmad), "\n")

## MAD vary between 0 (for variable 9) and 34. Therefore exclude
##  variable 9 and divide the remaining variables by their MADs.
bus1 <- bus[, -9]
madbus <- apply(bus1, 2, mad)
bus2 <- sweep(bus1, 2, madbus, "/", check.margin = FALSE)


pcaHubert <- PcaHubert(bus2, k=5, mcd=FALSE, alpha=0.5)
plot(pcaHubert, id.n.od=5, off=0.04, main="ROBPCA")

## Example 2: Glass spectra data

## 1. Load the Glass data from the package 'rospca'
library(rospca)
data(Glass)
dim(Glass)

##  The original data are not centered. Later we will center the spectra
##  by subtracting the univariate MCD location estimator from each wavelength.
##  Doing so allow us to observe more of the variability that is
##  present in the data.

plot(1:500, Glass[1, 1:500], pch="", xlab="Index", main="Original data", ylim=c(min(Glass), max(Glass)))
for(i in 1:nrow(Glass))
    lines(1:500, Glass[i,1:500], type="l", lwd=1.5, col="blue")

##================================================================

##  2. Instead of working with the raw data, we first robustly centered
##  the spectra by subtracting the univariate MCD location estimator
##  from each wavelength. Doing so allow us to observe more of the
##  variability that is present in the data.
##
##  We define a function to calculate the univariate MCD using the
##  internal rrcov function unimcd() and apply it to the data set
##  using the robustbase (undocumented) function doScale(). We set
##  the scale parameter to NULL - in order to do no scaling note
##  that in base::scale() we have scale=FALSE for this purpose.
##
##  Furthermore, instead of working with all 750 wavelengths, we will
##  take only the first 500 (the rest are not interesting).

umcd_center <- function(y)
    rrcov:::unimcd(y, quan=h.alpha.n(0.5, length(y), 1))$tmcd

X <- doScale(Glass, center=umcd_center, scale=NULL)$x
X <- X[, 1:500]
p <- ncol(X)
n <- nrow(X)

##  Plot all spectra data
##  plot(1:p, X[1, ], type="n", xlab="Wavenumber", ylab="Intensity", main="Original data", ylim=c(min(X), max(X)))
##  for(i in 1:n)
##      lines(1:p, X[i,], type="l", lwd=1.5, col="blue")

matplot(t(X), type="l", lty=1)

## These are the (known) groups of bad observations
xa <- c(57:63, 74:76)               # bad leverage points 5763 and 7476; grp==4
xb <- 143:179                       # bad leverage points
xc <- c(22, 23, 30)                 # orthogonal outliers 22, 23, 30
xd <- 180                           # borderline case
xgood <- 1:nrow(X)
xgood <- xgood[which(!(xgood %in% xa))]
xgood <- xgood[which(!(xgood %in% xb))]
xgood <- xgood[which(!(xgood %in% xc))]
xgood <- xgood[which(!(xgood %in% xd))]

## Plot the spectra of the different types of observations
matplot(t(X[xgood,]), type="l", lty=1, ylim=c(min(X), max(X)), ylab="")     # Regular observations
matplot(t(X[xb,]), type="l", lty=1, ylim=c(min(X), max(X)), ylab="")        # Bad leverage points 143179
matplot(t(X[xa,]), type="l", lty=1, ylim=c(min(X), max(X)), ylab="")        # Bad leverage points 5763 and 7476
matplot(t(X[xc,]), type="l", lty=1, ylim=c(min(X), max(X)), ylab="")        # orthogonal outliers 22, 23, 30


##===============================================================
##
##  How many components

pc <- PcaClassic(X)
summary(pc)

rpc <- PcaHubert(X, k=100, kmax=180, alpha=0.7, scale=FALSE)
summary(rpc)

rpc <- PcaHubert(X, k=10, kmax=180, alpha=0.5)
summary(rpc)

##===============================================================
## Classic PCA
pc <- PcaClassic(X, k=3, scale=FALSE)

plot(pc, id.n.sd=9, id.n.od=6)                  # outlier map
pca.scoreplot(pc)                               # score plot

## Loadings plot
matplot(pc$loadings, type="l", lty=1, lwd=2, ylab="", col=c(2,3,4), main="Classical PCA")
text(x=c(140, 300, 380), y=c(0.25, 0.2, 0.25), label=c(1,2,3))

##===============================================================
##  Robust PCA (ROBPCA)

rpc <- PcaHubert(X, k=3, scale=FALSE, alpha=0.7)

plot(rpc, id.n.sd=46, id.n.od=50)       # outlier map
which(rpc$od > rpc$cutoff.od)

pca.scoreplot(rpc)                      # score plot

## Loadings plot
matplot(rpc$loadings, type="l", lty=1, lwd=2, ylab="", col=c(2,3,4), main="ROBPCA")
text(x=c(140, 300, 380), y=c(0.25, 0.2, 0.25), label=c(3,1,2))


rpc.grid <- PcaGrid(X, k=4, scale=FALSE, alpha=0.5)
plot(rpc.grid, id.n.od=49)

which(rpc.grid$od > rpc.grid$cutoff.od)


##===============================================================
## Let us see now the Spherical PCA - they seem not to work properly with
##  p > n - the original code of Maronna et al. uses princomp,
##  I have repalced it by SVD (PcaClassic) but it seems, this is wrong.

rpc <- PcaHubert(X, k=3, scale=FALSE, alpha=0.7)
plot(rpc, id.n.sd=46, id.n.od=50, main="ROBPCA")
savePlot(file="robpca.pdf", type="pdf")

rpc.sph <- PcaLocantore(X, k=4, scale=FALSE)
plot(rpc.sph, id.n.od=49, main="Spherical PCA")
savePlot(file="spherical.pdf", type="pdf")
